/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.common;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Map;

import org.eclipse.cosmos.rm.validation.internal.launcher.ResourceValidation;

public class FileOutput extends TextOutput
{
	/**
	 * The default file name used to write the validation results
	 * to.
	 */
	private static final String DEFAULT_PATH = "validation-results";
	
	/**
	 * The file that will be written to
	 */
	private File file;
	
	/**
	 * The buffered writer
	 */
	private BufferedWriter bufferedWriter;
	
	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.common.AbstractValidationOutput#initialize(java.util.Map)
	 */
	public void initialize(Map attributes)
	{
		String filePath = (String)attributes.get(ResourceValidation.ATTRIBUTE_FILE_OUTPUT);
		final String DEFAUL_NAME =  DEFAULT_PATH + "-" + System.currentTimeMillis() + ".txt";
		if (filePath == null)
			filePath = DEFAUL_NAME;
		
		OutputStream fileOutput;
		try
		{
			file = new File(filePath);
			fileOutput = new FileOutputStream(filePath);
		} 
		catch (FileNotFoundException e)
		{			
			try
			{
				/* Try the default file path */				
				file = new File(DEFAUL_NAME);
				fileOutput = new FileOutputStream(DEFAUL_NAME);
			} catch (FileNotFoundException e1)
			{
				/* Print to system output */
				file = null;
				fileOutput = System.out;
			}
		}
		
		bufferedWriter = new BufferedWriter(new OutputStreamWriter(fileOutput));
	}
	
	public String getFilePath()
	{
		if (file == null)
			return SMLMessages.outputSystem;
		return file.getAbsolutePath();
	}

	protected void writeMessage(String message)
	{
		try
		{
			if (bufferedWriter == null)
				bufferedWriter = new BufferedWriter(new OutputStreamWriter(System.out));
			
			bufferedWriter.write(message + IValidationConstants.LINE_SEPARATOR);
			bufferedWriter.flush();
		} catch (IOException e)
		{
			file = null;
			bufferedWriter = new BufferedWriter(new OutputStreamWriter(System.out));			
		}				
	}

	public void close()
	{
		try
		{
			bufferedWriter.close();
		} catch (IOException e)
		{
			/* Ignore the error */
		}
	}

}
